@@ -24,6 +24,8 @@
 
 #include "ForwardDecls.h"
 #include "OscapScannerBase.h"
+#include "Utils.h"
+
 
 class OscapScannerLocal : public OscapScannerBase
 {
@@ -35,10 +37,23 @@ class OscapScannerLocal : public OscapScannerBase
 
         virtual QStringList getCommandLineArgs() const;
         virtual void evaluate();
+        /**
+         * @brief Return the executable name to execute and adjusts args if neccessary
+         * (e.g. the executable is a launcher and s.a. 'nice' and the oscap itself
+         * has to be added as an argument to it, i.e. prepended to args)
+         *
+         * @returns false when there is nothing to be read, true otherwise
+         * @see readStdOut
+         */
+        static QString getOscapProgramAndAdaptArgs(QStringList& args);
 
     private:
         static QString getPkexecOscapPath();
+        void fillInCapabilities();
 
+        void evaluateWithOfflineRemediation();
+        void evaluateWithOtherSettings();
+        static void setFilenameToTempFile(SpacelessQTemporaryFile& file);
 };
 
 #endif
