@@ -34,11 +34,16 @@
 #include "RPMOpenHelper.h"
 #include "Utils.h"
 #include "SSGIntegrationDialog.h"
+#include "RemediationRoleSaver.h"
 
 #include <QFileDialog>
+#include <QPushButton>
 #include <QAbstractEventDispatcher>
 #include <QCloseEvent>
+#include <QFileSystemWatcher>
 #include <QDesktopWidget>
+#include <QMenu>
+#include <QScreen>
 
 #include <cassert>
 #include <set>
@@ -83,6 +88,9 @@ MainWindow::MainWindow(QWidget* parent):
 
     mIgnoreProfileComboBox(false),
 
+    mFSWatch(new QFileSystemWatcher()),
+    mFSLastSeen(""),
+
     mRuleResultsExpanded(false)
 {
     mUI.setupUi(this);
@@ -123,6 +131,10 @@ MainWindow::MainWindow(QWidget* parent):
         mUI.actionOpenCustomizationFile, SIGNAL(triggered()),
         this, SLOT(openCustomizationFile())
     );
+    QObject::connect(
+        mUI.actionReloadContent, SIGNAL(triggered()),
+        this, SLOT(reloadContent())
+    );
     QObject::connect(
         mUI.checklistComboBox, SIGNAL(currentIndexChanged(int)),
         this, SLOT(checklistComboboxChanged(int))
@@ -216,6 +228,7 @@ MainWindow::MainWindow(QWidget* parent):
 
     mDiagnosticsDialog = new DiagnosticsDialog(this);
     mDiagnosticsDialog->hide();
+    globalDiagnosticsDialog = mDiagnosticsDialog;
 
     mCommandLineArgsDialog = new CommandLineArgsDialog(this);
     mCommandLineArgsDialog->hide();
@@ -245,12 +258,55 @@ MainWindow::MainWindow(QWidget* parent):
 
     closeFile();
 
-    // start centered
-    move(QApplication::desktop()->screen()->rect().center() - rect().center());
+    /* start centered
+
+    As of https://codereview.qt.nokia.com/c/qt/qtbase/+/268417,
+    QWidget *QDesktopWidget::screen(int) or Application::desktop()->screen() has been
+    deprecated with no replacement. A replacement might be provided in QT6 which
+    the following 4 lines of code should be re-evaluated if a replacement is provided.
+
+    The solution of the following 4 lines of code was indirectly provided by
+    Riccardo Fagiolo in "window_main.cpp" at
+    https://stackoverflow.com/questions/46300065/dynamically-resizing-two-qlabel-implementations
+    */
+    QSize size = QGuiApplication::screens().at(0)->availableSize();
+    int x = size.width() / 2 - width() / 2;
+    int y = size.height() / 2 - height() / 2;
+    move(x, y);
+    // End stackoverflow provided code by Riccardo Fagiolo
+
+    QAction* genBashRemediation = new QAction("&bash", this);
+    QObject::connect(
+        genBashRemediation, SIGNAL(triggered()),
+        this, SLOT(generateBashRemediationRole())
+    );
+    QAction* genAnsibleRemediation = new QAction("&ansible", this);
+    QObject::connect(
+        genAnsibleRemediation, SIGNAL(triggered()),
+        this, SLOT(generateAnsibleRemediationRole())
+    );
+    QAction* genPuppetRemediation = new QAction("&puppet", this);
+    QObject::connect(
+        genPuppetRemediation, SIGNAL(triggered()),
+        this, SLOT(generatePuppetRemediationRole())
+    );
+
+    QMenu* remediationButtonMenu = new QMenu(this);
+    remediationButtonMenu->addAction(genBashRemediation);
+    remediationButtonMenu->addAction(genAnsibleRemediation);
+    remediationButtonMenu->addAction(genPuppetRemediation);
+    mUI.genRemediationButton->setMenu(remediationButtonMenu);
+
+    QObject::connect(
+        mFSWatch, SIGNAL(fileChanged(const QString&)),
+        this, SLOT(fileChanged(const QString&))
+    );
 }
 
 MainWindow::~MainWindow()
 {
+    globalDiagnosticsDialog = NULL;
+
     delete mScanner;
     mScanner = 0;
 
@@ -261,6 +317,9 @@ MainWindow::~MainWindow()
 
     delete mQSettings;
     mQSettings = 0;
+
+    delete mFSWatch;
+    mFSWatch = NULL;
 }
 
 void MainWindow::setSkipValid(bool skipValid)
@@ -306,7 +365,7 @@ void MainWindow::clearResults()
     mUI.actionOpen->setEnabled(true);
 }
 
-void MainWindow::openFile(const QString& path)
+void MainWindow::openFile(const QString& path, bool reload)
 {
     try
     {
@@ -321,7 +380,7 @@ void MainWindow::openFile(const QString& path)
         }
 
         mScanningSession->setSkipValid(mSkipValid);
-        mScanningSession->openFile(inputPath);
+        mScanningSession->openFile(inputPath, reload);
 
         // In case openscap autonegotiated opening a tailoring file directly
         if (tailoringPath.isEmpty() && mScanningSession->hasTailoring())
@@ -348,6 +407,14 @@ void MainWindow::openFile(const QString& path)
 
         centralWidget()->setEnabled(true);
 
+        // Refill mFSWatch after opening file
+        mFSWatch->removePaths(mFSWatch->files());
+        for (const QString path : mScanningSession->getOriginalClosure())
+        {
+            mFSWatch->addPath(path);
+        }
+        mFSLastSeen = "";
+
         mDiagnosticsDialog->infoMessage(QObject::tr("Opened file '%1'.").arg(path));
     }
     catch (const std::exception& e)
@@ -375,6 +442,9 @@ void MainWindow::openFileDialog()
     if (!QFileInfo(defaultDirectory).isDir())
         defaultDirectory = "";
 
+    // Save the current file path to restore if we're unable to open a valid file
+    const QString currentFile = getOpenedFilePath();
+
     bool opened = false;
     while (!opened)
     {
@@ -393,7 +463,7 @@ void MainWindow::openFileDialog()
 
         if (fileOpened())
         {
-            if (openNewFileQuestionDialog(mScanningSession->getOpenedFilePath()) == QMessageBox::Yes)
+            if (openNewFileQuestionDialog(mScanningSession->getOriginalFilePath()) == QMessageBox::Yes)
                 closeFile();
             else
                 // user cancelled closing current file, we have to abort
@@ -414,7 +484,19 @@ void MainWindow::openFileDialog()
 
     if (!fileOpened())
     {
-        if (!close())
+        if (currentFile != "")
+        {
+            openFile(currentFile);
+
+            if (!fileOpened())
+            {
+                // Error occurred, keep pumping events and don't move on until user
+                // dismisses diagnostics dialog.
+                mDiagnosticsDialog->waitUntilHidden();
+            }
+        }
+
+        if (!fileOpened() && !close())
             throw MainWindowException("Failed to close main window!");
     }
 }
@@ -447,7 +529,7 @@ void MainWindow::openSSGDialog(const QString& customDismissLabel)
 
             if (fileOpened())
             {
-                if (openNewFileQuestionDialog(mScanningSession->getOpenedFilePath()) == QMessageBox::Yes)
+                if (openNewFileQuestionDialog(mScanningSession->getOriginalFilePath()) == QMessageBox::Yes)
                 {
                     closeFile();
                 }
@@ -471,6 +553,35 @@ void MainWindow::openSSGDialog(const QString& customDismissLabel)
     delete dialog;
 }
 
+void MainWindow::openTailoringFile(const QString& path)
+{
+    if (!fileOpened())
+        throw MainWindowException("Can't load a tailoring file, SCAP input hasn't been loaded yet.");
+
+    mScanningSession->setTailoringFile(path);
+    markLoadedTailoringFile(path);
+    reloadSession();
+    refreshTailoringProfiles();
+}
+
+void MainWindow::reloadContent()
+{
+    const QString currentFile = mScanningSession->getOriginalFilePath();
+    openFile(currentFile, true);
+
+    if (!fileOpened())
+    {
+        // Error occurred, keep pumping events and don't move on until user
+        // dismisses diagnostics dialog.
+        mDiagnosticsDialog->waitUntilHidden();
+
+        if (!close())
+        {
+            throw MainWindowException("Failed to close main window!");
+        }
+    }
+}
+
 void MainWindow::closeMainWindowAsync()
 {
     emit closeMainWindow();
@@ -528,6 +639,7 @@ void MainWindow::scanAsync(ScannerMode scannerMode)
 
     mUI.menuSave->setEnabled(false);
     mUI.actionOpen->setEnabled(false);
+    mUI.actionReloadContent->setEnabled(false);
 
     mUI.ruleResultsTree->prepareForScanning();
 
@@ -546,6 +658,7 @@ void MainWindow::scanAsync(ScannerMode scannerMode)
 
         mUI.menuSave->setEnabled(true);
         mUI.actionOpen->setEnabled(true);
+        mUI.actionReloadContent->setEnabled(true);
 
         return;
     }
@@ -811,7 +924,7 @@ void MainWindow::refreshProfiles()
             mUI.profileComboBox->addItem(profileTitle, QVariant(it->first));
         }
 
-        if (previouslySelected != QString::Null())
+        if (!previouslySelected.isNull())
         {
             const int indexCandidate = mUI.profileComboBox->findData(QVariant(previouslySelected));
             if (indexCandidate != -1)
@@ -1042,21 +1155,7 @@ void MainWindow::tailoringFileComboboxChanged(int index)
     reloadSession();
     if (tailoringLoaded)
     {
-        const std::map<QString, struct xccdf_profile*> profiles = mScanningSession->getAvailableProfiles();
-
-        // Select the first tailored profile from the newly loaded tailoring
-        for (std::map<QString, struct xccdf_profile*>::const_iterator it = profiles.begin();
-             it != profiles.end(); ++it)
-        {
-            if (xccdf_profile_get_tailoring(it->second))
-            {
-                const QString profileId = it->first;
-                const int idx = mUI.profileComboBox->findData(QVariant(profileId));
-                if (idx != -1)
-                    mUI.profileComboBox->setCurrentIndex(idx);
-                break;
-            }
-        }
+        refreshTailoringProfiles();
     }
 
     mOldTailoringComboBoxIdx = index;
@@ -1090,6 +1189,25 @@ void MainWindow::profileComboboxChanged(int index)
     clearResults();
 }
 
+void MainWindow::refreshTailoringProfiles()
+{
+    const std::map<QString, struct xccdf_profile*> profiles = mScanningSession->getAvailableProfiles();
+
+    // Select the first tailored profile from the newly loaded tailoring
+    for (std::map<QString, struct xccdf_profile*>::const_iterator it = profiles.begin();
+         it != profiles.end(); ++it)
+    {
+        if (xccdf_profile_get_tailoring(it->second))
+        {
+            const QString profileId = it->first;
+            const int idx = mUI.profileComboBox->findData(QVariant(profileId));
+            if (idx != -1)
+                mUI.profileComboBox->setCurrentIndex(idx);
+            break;
+        }
+    }
+}
+
 void MainWindow::toggleRuleResultsExpanded()
 {
     mRuleResultsExpanded = !mRuleResultsExpanded;
@@ -1167,7 +1285,7 @@ void MainWindow::scanWarningMessage(const QString& message)
     mDiagnosticsDialog->warningMessage(message);
 }
 
-void MainWindow::scanErrorMessage(const QString &message)
+void MainWindow::scanErrorMessage(const QString& message)
 {
     mDiagnosticsDialog->errorMessage(message);
 }
@@ -1217,6 +1335,7 @@ void MainWindow::scanEnded(bool canceled)
 
     mUI.menuSave->setEnabled(true);
     mUI.actionOpen->setEnabled(true);
+    mUI.actionReloadContent->setEnabled(true);
 
     cleanupScanThread();
 }
@@ -1449,6 +1568,29 @@ bool MainWindow::unsavedTailoringChanges() const
     return mUI.tailoringFileComboBox->currentIndex() == idx;
 }
 
+void MainWindow::fileChanged(const QString& path)
+{
+    if (path == mFSLastSeen)
+        return;
+    mFSLastSeen = path;
+
+    QMessageBox question;
+    question.setText(QObject::tr("The following file was modified after opening: %1").arg(path));
+    question.setInformativeText(QObject::tr("Do you wish to reload it, losing any unsaved tailoring changes?"));
+
+    QPushButton *reload = question.addButton(QObject::tr("Reload"), QMessageBox::AcceptRole);
+    QPushButton *ignore = question.addButton(QObject::tr("Ignore"), QMessageBox::RejectRole);
+    question.setDefaultButton(ignore);
+    question.setIcon(QMessageBox::Question);
+
+    question.exec();
+
+    if (question.clickedButton() == reload)
+    {
+        reloadContent();
+    }
+}
+
 QString MainWindow::getDefaultSaveDirectory()
 {
     return mQSettings->value("last_save_directory", "").toString();
@@ -1510,8 +1652,27 @@ QMessageBox::StandardButton MainWindow::openNewFileQuestionDialog(const QString&
 {
     return QMessageBox::question(this,
           QObject::tr("Close currently opened file?"),
-          QObject::tr("Opened file '%1' has to be closed before opening another file.\n\n"
+          QObject::tr("Opened file '%1' has to be closed before opening another file. "
+          "Closing this file will lose any unsaved tailoring.\n\n"
           "Do you want to proceed?").arg(oldFilepath),
           QMessageBox::Yes | QMessageBox::No, QMessageBox::No
     );
 }
+
+void MainWindow::generateBashRemediationRole()
+{
+    BashProfileRemediationSaver saver(this, mScanningSession);
+    saver.selectFilenameAndSaveRole();
+}
+
+void MainWindow::generateAnsibleRemediationRole()
+{
+    AnsibleProfileRemediationSaver saver(this, mScanningSession);
+    saver.selectFilenameAndSaveRole();
+}
+
+void MainWindow::generatePuppetRemediationRole()
+{
+    PuppetProfileRemediationSaver saver(this, mScanningSession);
+    saver.selectFilenameAndSaveRole();
+}
