@@ -36,6 +36,7 @@
 #include <QTimer>
 #include <QDateTime>
 #include <QStack>
+#include <QScreen>
 
 #include <algorithm>
 #include <cassert>
@@ -76,8 +77,8 @@ TailoringWindow::TailoringWindow(struct xccdf_policy* policy, struct xccdf_bench
 
     mSearchSkippedItems(0),
     mSearchCurrentNeedle(""),
-    
-    
+
+    mSelectAllAction(new QAction(this)),
     mDeselectAllAction(new QAction(this))
 {
     generateValueAffectsRulesMap(xccdf_benchmark_to_item(benchmark));
@@ -131,11 +132,11 @@ TailoringWindow::TailoringWindow(struct xccdf_policy* policy, struct xccdf_bench
 
     QObject::connect(
         mUI.itemsTree, SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)),
-        this, SLOT(itemSelectionChanged(QTreeWidgetItem*,QTreeWidgetItem*))
+        this, SLOT(itemSelectionChanged(QTreeWidgetItem*))
     );
     QObject::connect(
-        mUI.itemsTree, SIGNAL(itemChanged(QTreeWidgetItem*,int)),
-        this, SLOT(itemChanged(QTreeWidgetItem*,int))
+        mUI.itemsTree, SIGNAL(itemChanged(QTreeWidgetItem*, int)),
+        this, SLOT(itemChanged(QTreeWidgetItem*))
     );
     QObject::connect(
         mUI.itemsTree, SIGNAL(itemExpanded(QTreeWidgetItem*)),
@@ -162,7 +163,7 @@ TailoringWindow::TailoringWindow(struct xccdf_policy* policy, struct xccdf_bench
     createTreeItem(mBenchmarkItem, xccdf_benchmark_to_item(mBenchmark));
     synchronizeTreeItem();
 
-    mUI.itemsTree->header()->setResizeMode(0, QHeaderView::ResizeToContents);
+    mUI.itemsTree->header()->setSectionResizeMode(0, QHeaderView::ResizeToContents);
     mUI.itemsTree->header()->setStretchLastSection(false);
 
     deserializeCollapsedItems();
@@ -186,6 +187,13 @@ TailoringWindow::TailoringWindow(struct xccdf_policy* policy, struct xccdf_bench
 
     mUI.toolBar->addSeparator();
 
+    mSelectAllAction->setText(QObject::tr("Select All"));
+    QObject::connect(
+        mSelectAllAction, SIGNAL(triggered()),
+        this, SLOT(selectAllChildrenItems())
+    );
+    mUI.toolBar->addAction(mSelectAllAction);
+
     mDeselectAllAction->setText(QObject::tr("Deselect All"));
     QObject::connect(
         mDeselectAllAction, SIGNAL(triggered()),
@@ -223,8 +231,22 @@ TailoringWindow::TailoringWindow(struct xccdf_policy* policy, struct xccdf_bench
         this, SLOT(searchNext())
     );
 
-    // start centered
-    move(QApplication::desktop()->screen()->rect().center() - rect().center());
+    /* start centered
+
+    As of https://codereview.qt.nokia.com/c/qt/qtbase/+/268417,
+    QWidget *QDesktopWidget::screen(int) or Application::desktop()->screen() has been
+    deprecated with no replacement. A replacement might be provided in QT6 which
+    the following 4 lines of code should be re-evaluated if a replacement is provided.
+
+    The solution of the following 4 lines of code was indirectly provided by
+    Riccardo Fagiolo in "window_main.cpp" at
+    https://stackoverflow.com/questions/46300065/dynamically-resizing-two-qlabel-implementations
+    */
+    QSize size = QGuiApplication::screens().at(0)->availableSize();
+    int x = size.width() / 2 - width() / 2;
+    int y = size.height() / 2 - height() / 2;
+    move(x, y);
+    // End stackoverflow provided code by Riccardo Fagiolo
     show();
 }
 
@@ -264,20 +286,31 @@ void TailoringWindow::synchronizeProfileItem()
 
 void TailoringWindow::synchronizeTreeItem()
 {
+    // If itemsTree has focus, it will lose it when it becomes invisible
+    QWidget* focused = QApplication::focusWidget();
+
     // If QTreeWidget remains visible during the sync, it'll recalculate its geometry for each checked
     // row. Configuring "visible" as false seems to be the only way to avoid entering the "if" in
     // https://github.com/qt/qt/blob/4.8/src/gui/itemviews/qabstractitemview.cpp#L3190
     mUI.itemsTree->setVisible(false);
     synchronizeTreeItemSelections(mBenchmarkItem);
     mUI.itemsTree->setVisible(true);
-    
+
+    // So we set the focus to the widget again
+    if (focused)
+        focused->setFocus();
+
     // Enables/disables "Deselect All action based on weather the top level rules/groups are checked"
     bool anySelected = false;
+    bool anyNotSelected = false;
     for (int i = 0; i < mBenchmarkItem->childCount(); ++i)
     {
-        anySelected |= mBenchmarkItem->child(i)->checkState(0) != Qt::Unchecked;
+        const bool isSelected = mBenchmarkItem->child(i)->checkState(0) != Qt::Unchecked;
+        anySelected |= isSelected;
+        anyNotSelected |= !isSelected;
     }
     mDeselectAllAction->setEnabled(anySelected);
+    mSelectAllAction->setEnabled(anyNotSelected);
 }
 
 void TailoringWindow::synchronizeTreeItemSelections(QTreeWidgetItem* treeItem)
@@ -326,8 +359,8 @@ void TailoringWindow::synchronizeTreeItemSelections(QTreeWidgetItem* treeItem)
                 }
                 break;
             }
-            //fall through
         }
+        //fallthrough
         case XCCDF_RULE:
             treeItem->setCheckState(0,
                     getXccdfItemInternalSelected(mPolicy, xccdfItem) ? Qt::Checked : Qt::Unchecked);
@@ -420,6 +453,11 @@ const std::vector<struct xccdf_rule*>& TailoringWindow::getRulesAffectedByValue(
     return empty;
 }
 
+void TailoringWindow::selectAllChildrenItems()
+{
+    createSelectionMacro(mBenchmarkItem, true, QObject::tr("Select All"));
+}
+
 void TailoringWindow::deselectAllChildrenItems()
 {
     createSelectionMacro(mBenchmarkItem, false, QObject::tr("Deselect All"));
@@ -534,7 +572,7 @@ void TailoringWindow::deleteProfileAndDiscard()
     close();
 }
 
-void TailoringWindow::closeEvent(QCloseEvent * event)
+void TailoringWindow::closeEvent(QCloseEvent* event)
 {
     if (!mChangesConfirmed)
     {
@@ -665,6 +703,9 @@ void TailoringWindow::syncCollapsedItem(QTreeWidgetItem* item, QSet<QString>& us
     struct xccdf_item* xccdfItem = getXccdfItemFromTreeItem(item);
     const QString id = QString::fromUtf8(xccdf_item_get_id(xccdfItem));
 
+    for (int i = 0; i < item->childCount(); ++i)
+        syncCollapsedItem(item->child(i), usedCollapsedIds);
+
     if (mCollapsedItemIds.contains(id))
     {
         mUI.itemsTree->collapseItem(item);
@@ -674,9 +715,6 @@ void TailoringWindow::syncCollapsedItem(QTreeWidgetItem* item, QSet<QString>& us
     {
         mUI.itemsTree->expandItem(item);
     }
-
-    for (int i = 0; i < item->childCount(); ++i)
-        syncCollapsedItem(item->child(i), usedCollapsedIds);
 }
 
 void TailoringWindow::createTreeItem(QTreeWidgetItem* treeItem, struct xccdf_item* xccdfItem)
@@ -892,7 +930,7 @@ void TailoringWindow::searchNext()
             // We cannot use setCurrentItem() on disabled items
             // so we will use a workaround
 
-            QTreeWidgetItem * dummyItem = mUI.itemsTree->currentItem();
+            QTreeWidgetItem* dummyItem = mUI.itemsTree->currentItem();
 
             // Setting of "new" current item cause removing selection
             // from rest of items
@@ -904,7 +942,7 @@ void TailoringWindow::searchNext()
             // Emulate setting of "currentItem"
             match->setSelected(true);
             mUI.itemsTree->scrollToItem(match);
-            emit itemSelectionChanged(match, NULL);
+            emit itemSelectionChanged(match);
         }
 
         mSearchBox->setStyleSheet("");
@@ -919,7 +957,7 @@ void TailoringWindow::searchNext()
     }
 }
 
-void TailoringWindow::itemSelectionChanged(QTreeWidgetItem* current, QTreeWidgetItem* previous)
+void TailoringWindow::itemSelectionChanged(QTreeWidgetItem* current)
 {
     struct xccdf_item* item = getXccdfItemFromTreeItem(current);
     setUpdatesEnabled(false);
@@ -938,7 +976,7 @@ void TailoringWindow::itemSelectionChanged(QTreeWidgetItem* current, QTreeWidget
     setUpdatesEnabled(true);
 }
 
-void TailoringWindow::itemChanged(QTreeWidgetItem* treeItem, int column)
+void TailoringWindow::itemChanged(QTreeWidgetItem* treeItem)
 {
     if (mSynchronizeItemLock > 0)
         return;
